##
## NOTE: The code below is only for illustration purposes.
##       ANY CHANGES HERE WILL NOT BE REFLECTED IN THE EVALUATION SETUP
##       PLEASE MAKE YOUR CHANGES IN `run.py`.
##

import numpy as np

from evaluator.dataset import ZEWDPCBaseDataset, ZEWDPCProtectedDataset
from run import ZEWDPCBaseRun

####################################################################################
####################################################################################
##
## Dataset Initialization
####################################################################################
DATASET_SHUFFLE_SEED = 1022022

# Instantiate Training Dataset
training_dataset = ZEWDPCBaseDataset(
    images_dir="./data/prepared/v0.1/dataset_debug",
    labels_path="./data/prepared/v0.1/dataset_debug/labels.csv",
    shuffle_seed=DATASET_SHUFFLE_SEED,
)
# Instantiate Unlabelled Dataset
unlabelled_dataset = ZEWDPCProtectedDataset(
    images_dir="./data/prepared/v0.1/dataset_debug",
    labels_path="./data/prepared/v0.1/dataset_debug/labels.csv",
    budget=100,  # Configurable Parameter
    shuffle_seed=DATASET_SHUFFLE_SEED,
)
# Instantiate Validation Dataset
val_dataset = ZEWDPCBaseDataset(
    images_dir="./data/prepared/v0.1/dataset_debug",
    labels_path="./data/prepared/v0.1/dataset_debug/labels.csv",
    drop_labels=True,
    shuffle_seed=DATASET_SHUFFLE_SEED,
)
# A second instantiation of the validation test with the labels present
#       - helpful later, when computing the scores.
val_dataset_gt = ZEWDPCBaseDataset(
    images_dir="./data/prepared/v0.1/dataset_debug",
    labels_path="./data/prepared/v0.1/dataset_debug/labels.csv",
    drop_labels=False,
    shuffle_seed=DATASET_SHUFFLE_SEED,
)

####################################################################################
####################################################################################
##
## Phase 1 : Pre-Training Phase
####################################################################################
run = ZEWDPCBaseRun()
run.pre_training_phase(training_dataset)

run.save_checkpoint("/tmp/pretrainig_phase_checkpoint.pickle")
# NOTE:It is critical that the checkpointing works in a self-contained way
#       As, the evaluators might choose to run the different phases separately.

del run

####################################################################################
####################################################################################
##
## Phase 2 : Purchase Phase
####################################################################################
run = ZEWDPCBaseRun()
run.load_checkpoint("/tmp/pretrainig_phase_checkpoint.pickle")
run.purchase_phase(unlabelled_dataset, training_dataset, budget=3000)

run.save_checkpoint("/tmp/purchase_phase_checkpoint.pickle")
del run

####################################################################################
####################################################################################
##
## Phase 3 : Prediction Phase
####################################################################################
run = ZEWDPCBaseRun()
run.load_checkpoint("/tmp/purchase_phase_checkpoint.pickle")
predictions = run.prediction_phase(val_dataset)
assert type(predictions) == np.ndarray
assert predictions.shape == (len(val_dataset), 4)

####################################################################################
####################################################################################
##
## Phase 4 : Evaluation Phase
####################################################################################
from evaluator.evaluation_metrics import accuracy_score, hamming_loss, exact_match_ratio

y_true = val_dataset_gt._get_all_labels()
y_pred = predictions

accuracy_score = accuracy_score(y_true, y_pred)
hamming_loss_score = hamming_loss(y_true, y_pred)
exact_match_ratio_score = exact_match_ratio(y_true, y_pred)

print("Accuracy Score : ", accuracy_score)
print("Hamming Loss : ", hamming_loss_score)
print("Exact Match Ratio : ", exact_match_ratio_score)
